;;; rt-test.el --- RT tests for RT itself

;; Copyright (C) 2010 Didier Verna
;; Copyright (C) 1990 Massachusetts Institute of Technology, Cambridge MA.

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>
;; Created:       Tue Dec  8 11:38:45 2009
;; Last Revision: Mon Feb 15 17:07:32 2010
;; Keywords:      lisp


;; Permission to use, copy, modify, and distribute this software and its
;; documentation for any purpose and without fee is hereby granted, provided
;; that this copyright and permission notice appear in all copies and
;; supporting documentation, and that the name of M.I.T. not be used in
;; advertising or publicity pertaining to distribution of the software without
;; specific, written prior permission. M.I.T. makes no representations about
;; the suitability of this software for any purpose. It is provided "as is"
;; without express or implied warranty.

;; M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
;; BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
;; ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
;; IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

;; This is the December 19, 1990 version of a set of tests that use the RT
;; regression tester to test itself. See the documentation of RT for a
;; discusion of how to use this file.


;;; Commentary:

;; Contents management by FCM version 0.1.

;; Symbols in this file are prefixed with rtt:.


;;; Code:

(require 'rt)


;;(defun rtt:normalize-stream (s)
;;  (let ((l nil))
;;    ;; #### FIXME:
;;    (loop (push (read-line s nil s) l)
;;      (when (eq (car l) s)
;;	(setq l (nreverse (cdr l)))
;;	(return nil)))
;;    (delete "" l :test #'equal)))
;;
;;(defun rtt:normalize (string)
;;  ;; #### FIXME:
;;  (with-input-from-string (s string)
;;			  (rtt:normalize-stream s)))

(defun normalize (string)
  (remove "" (split-string string "\n")))

(defun rtt:do-setup (form)
  (let ((rt:*test* nil)
	(rt:*do-tests-when-defined* nil)
	(rt::*entries* (list nil))
	(rt::*in-test* nil)
	(rt::*debug* t)
	result)
    (rt:deftest t1 4 4)
    (rt:deftest (t 2) 4 3)
    (let ((buffer (get-buffer-create "*RT*")))
      (erase-buffer buffer)
      (setq result (multiple-value-list (catch 'rt::*debug* (eval form))))
      (values-list
       (cons (normalize (buffer-string buffer)) result)))))

(defmacro rtt:setup (&rest body)
  `(rtt:do-setup '(progn ., body)))



;;(defvar rtt:*file-name* nil)
;;
;;(defun rtt:get-file-name ()
;;  (loop (if rtt:*file-name*
;;	    (return rtt:*file-name*))
;;    ;; #### FIXME:
;;    (format *error-output*
;;	    "~%Type a string representing naming of a scratch disk file: ")
;;    (setq rtt:*file-name* (read))
;;    (if (not (stringp rtt:*file-name*))
;;	(setq rtt:*file-name* nil))))
;;
;;(rtt:get-file-name)
;;
;;(defmacro rtt:with-temporary-file (f &body forms)
;;  `(let ((,f *file-name*))
;;    ,@ forms
;;    (rtt:get-file-output ,f)))
;;
;;(defun rtt:get-file-output (f)
;;  ;; #### FIXME:
;;  (prog1 (with-open-file (in f)
;;	   (rtt:normalize-stream in))
;;    (delete-file f)))


(rt:rem-all-tests)

(rt:deftest rt:deftest-1
    (rtt:setup (rt:deftest t1 3 3)
	       (values (rt:get-test 't1) rt:*test* (rt:pending-tests)))
  ("Redefining test t1.") (t1 3 3) t1 (t1 (t 2)))
(rt:deftest rt:deftest-2
    (rtt:setup (rt:deftest (t 2) 3 3)
	       (rt:get-test '(t 2)))
  ("Redefining test (t 2).") ((t 2) 3 3))
(rt:deftest rt:deftest-3
    (rtt:setup (rt:deftest 2 3 3)
	       (values (rt:get-test 2) rt:*test* (rt:pending-tests)))
  () (2 3 3) 2 (t1 (t 2) 2))
(rt:deftest rt:deftest-4
    (rtt:setup (let ((rt:*do-tests-when-defined* t))
		 (rt:deftest (temp) 4 3)))
  ("Test (temp) failed"
   "Form: 4"
   "Expected value(s): (3)"
   "Actual value(s):   (4)")
  (temp))
(rt:deftest do-test-1
    (rtt:setup (values (rt:do-test 't1) rt:*test* (rt:pending-tests)))
  () t1 t1 ((t 2)))
(rt:deftest do-test-2
    (rtt:setup (values (rt:do-test '(t 2)) (rt:pending-tests)))
  ("Test (t 2) failed"
   "Form: 4"
   "Expected value(s): (3)"
   "Actual value(s):   (4)")
  nil (t1 (t 2)))
(rt:deftest do-test-3
    (rtt:setup (let ((rt:*test* 't1)) (rt:do-test)))
  () t1)
(rt:deftest get-test-1
    (rtt:setup (values (rt:get-test 't1) rt:*test*))
  () (t1 4 4) (t 2))
(rt:deftest get-test-2
    (rtt:setup (rt:get-test '(t 2)))
  ()((t 2) 4 3))
(rt:deftest get-test-3
    (rtt:setup (let ((rt:*test* 't1)) (rt:get-test)))
  () (t1 4 4))
(rt:deftest get-test-4
    (rtt:setup (rt:deftest t3 1 1) (rt:get-test))
  () (t3 1 1))
(rt:deftest get-test-5
    (rtt:setup (rt:get-test 't0))
  ("No test with name t0.") nil)
(rt:deftest rem-test-1
    (rtt:setup (values (rt:rem-test 't1) (rt:pending-tests)))
  () t1 ((t 2)))
(rt:deftest rem-test-2
    (rtt:setup (values (rt:rem-test '(t 2)) (rt:pending-tests)))
  () (t 2) (t1))
(rt:deftest rem-test-3
    (rtt:setup (let ((rt:*test* '(t 2))) (rt:rem-test)) (rt:pending-tests))
  () (t1))
(rt:deftest rem-test-4
    (rtt:setup (values (rt:rem-test 't0) (rt:pending-tests)))
  ()  nil (t1 (t 2)))
(rt:deftest rem-test-5
    (rtt:setup (rt:rem-all-tests) (rt:rem-test 't0) (rt:pending-tests))
  () ())
(rt:deftest rem-all-tests-1
    (rtt:setup (values (rt:rem-all-tests) (rt:pending-tests)))
  () nil  nil)
(rt:deftest rem-all-tests-2
    (rtt:setup (rt:rem-all-tests) (rt:rem-all-tests) (rt:pending-tests))
  () nil)

;;(rt:deftest do-tests-1
;;    ;; #### FIXME:
;;    (rtt:setup (let ((*print-case* :downcase))
;;		 (values (do-tests) (continue-testing) (do-tests))))
;;  ("Doing 2 pending tests of 2 tests total."
;;   " T1"
;;   "Test (T 2) failed"
;;   "Form: 4"
;;   "Expected value: 3"
;;   "Actual value: 4."
;;   "1 out of 2 total tests failed: (T 2)."
;;   "Doing 1 pending test of 2 tests total."
;;   "Test (T 2) failed"
;;   "Form: 4"
;;   "Expected value: 3"
;;   "Actual value: 4."
;;   "1 out of 2 total tests failed: (T 2)."
;;   "Doing 2 pending tests of 2 tests total."
;;   " T1"
;;   "Test (T 2) failed"
;;   "Form: 4"
;;   "Expected value: 3"
;;   "Actual value: 4."
;;   "1 out of 2 total tests failed: (T 2).")
;;  nil
;;  nil
;;  nil)

(rt:deftest do-tests-2
    (rtt:setup (rt:rem-test '(t 2))
	       (rt:deftest (t 2) 3 3)
	       (values (rt:do-tests) (rt:continue-testing) (rt:do-tests)))
  ("Doing 2 pending test(s) of 2 test(s) total."
   " T1 (T 2)"
   "No tests failed."
   "Doing 0 pending test(s) of 2 test(s) total."
   "No tests failed."
   "Doing 2 pending test(s) of 2 test(s) total."
   " T1 (T 2)"
   "No tests failed.")
  t
  t
  t)

(rt:deftest do-tests-3
    (rtt:setup (rt:rem-all-tests) (values (rt:do-tests) (rt:continue-testing)))
  ("Doing 0 pending test(s) of 0 test(s) total."
   "No tests failed."
   "Doing 0 pending test(s) of 0 test(s) total."
   "No tests failed.")
  t
  t)

;;(rt:deftest do-tests-4
;;    (rtt:setup (normalize (with-output-to-string (s) (do-tests s))))
;;  ()
;;  ("Doing 2 pending tests of 2 tests total."
;;   " T1"
;;   "Test (T 2) failed"
;;   "Form: 4"
;;   "Expected value: 3"
;;   "Actual value: 4."
;;   "1 out of 2 total tests failed: (T 2)."))

;;(rt:deftest do-tests-5
;;    (rtt:setup (with-temporary-file s (do-tests s)))
;;  ()
;;  ("Doing 2 pending tests of 2 tests total."
;;   " T1"
;;   "Test (T 2) failed"
;;   "Form: 4"
;;   "Expected value: 3"
;;   "Actual value: 4."
;;   "1 out of 2 total tests failed: (T 2)."))

(rt:deftest continue-testing-1
    (rtt:setup (rt:deftest temp (rt:continue-testing) 5)
	       (rt:do-test 'temp) (rt:pending-tests))
  () (t1 (t 2) temp))


;;; rt-test.el ends here
